@@ -37,6 +37,7 @@ from phoenix.server.api.input_types.InvocationParameters import (
     InvocationParameter,
     InvocationParameterInput,
     JSONInvocationParameter,
+    StringInvocationParameter,
     StringListInvocationParameter,
     extract_parameter,
     validate_invocation_parameters,
@@ -466,7 +467,6 @@ class OpenAIBaseStreamingClient(PlaygroundStreamingClient):
         "gpt-3.5-turbo-0125",
         "gpt-3.5-turbo",
         "gpt-3.5-turbo-1106",
-        "gpt-3.5-turbo-instruct",
     ],
 )
 class OpenAIStreamingClient(OpenAIBaseStreamingClient):
@@ -477,11 +477,12 @@ class OpenAIStreamingClient(OpenAIBaseStreamingClient):
     ) -> None:
         from openai import AsyncOpenAI
 
-        # todo: check if custom base url is set before raising error to allow
-        # for custom endpoints that don't require an API key
+        base_url = model.base_url or os.environ.get("OPENAI_BASE_URL")
         if not (api_key := api_key or os.environ.get("OPENAI_API_KEY")):
-            raise BadRequest("An API key is required for OpenAI models")
-        client = AsyncOpenAI(api_key=api_key)
+            if not base_url:
+                raise BadRequest("An API key is required for OpenAI models")
+            api_key = "sk-fake-api-key"
+        client = AsyncOpenAI(api_key=api_key, base_url=base_url)
         super().__init__(client=client, model=model, api_key=api_key)
         self._attributes[LLM_PROVIDER] = OpenInferenceLLMProviderValues.OPENAI.value
         self._attributes[LLM_SYSTEM] = OpenInferenceLLMSystemValues.OPENAI.value
@@ -496,12 +497,19 @@ class OpenAIStreamingClient(OpenAIBaseStreamingClient):
         "o1-mini-2024-09-12",
         "o1-preview",
         "o1-preview-2024-09-12",
+        "o3-mini",
+        "o3-mini-2025-01-31",
     ],
 )
-class OpenAIO1StreamingClient(OpenAIStreamingClient):
+class OpenAIReasoningStreamingClient(OpenAIStreamingClient):
     @classmethod
     def supported_invocation_parameters(cls) -> list[InvocationParameter]:
         return [
+            StringInvocationParameter(
+                invocation_name="reasoning_effort",
+                label="Reasoning Effort",
+                canonical_name=CanonicalParameterName.REASONING_EFFORT,
+            ),
             IntInvocationParameter(
                 invocation_name="max_completion_tokens",
                 canonical_name=CanonicalParameterName.MAX_COMPLETION_TOKENS,
@@ -524,6 +532,49 @@ class OpenAIO1StreamingClient(OpenAIStreamingClient):
             ),
         ]
 
+    async def chat_completion_create(
+        self,
+        messages: list[
+            tuple[ChatCompletionMessageRole, str, Optional[str], Optional[list[JSONScalarType]]]
+        ],
+        tools: list[JSONScalarType],
+        **invocation_parameters: Any,
+    ) -> AsyncIterator[ChatCompletionChunk]:
+        from openai import NOT_GIVEN
+
+        # Convert standard messages to OpenAI messages
+        openai_messages = []
+        for message in messages:
+            openai_message = self.to_openai_chat_completion_param(*message)
+            if openai_message is not None:
+                openai_messages.append(openai_message)
+
+        throttled_create = self.rate_limiter._alimit(self.client.chat.completions.create)
+        response = await throttled_create(
+            messages=openai_messages,
+            model=self.model_name,
+            stream=False,
+            tools=tools or NOT_GIVEN,
+            **invocation_parameters,
+        )
+
+        if response.usage is not None:
+            self._attributes.update(dict(self._llm_token_counts(response.usage)))
+
+        choice = response.choices[0]
+        if choice.message.content:
+            yield TextChunk(content=choice.message.content)
+
+        if choice.message.tool_calls:
+            for tool_call in choice.message.tool_calls:
+                yield ToolCallChunk(
+                    id=tool_call.id,
+                    function=FunctionCallChunk(
+                        name=tool_call.function.name,
+                        arguments=tool_call.function.arguments,
+                    ),
+                )
+
     def to_openai_chat_completion_param(
         self,
         role: ChatCompletionMessageRole,
@@ -533,6 +584,7 @@ class OpenAIO1StreamingClient(OpenAIStreamingClient):
     ) -> Optional["ChatCompletionMessageParam"]:
         from openai.types.chat import (
             ChatCompletionAssistantMessageParam,
+            ChatCompletionDeveloperMessageParam,
             ChatCompletionToolMessageParam,
             ChatCompletionUserMessageParam,
         )
@@ -545,7 +597,12 @@ class OpenAIO1StreamingClient(OpenAIStreamingClient):
                 }
             )
         if role is ChatCompletionMessageRole.SYSTEM:
-            return None  # System messages are not supported for o1 models
+            return ChatCompletionDeveloperMessageParam(
+                {
+                    "content": content,
+                    "role": "developer",
+                }
+            )
         if role is ChatCompletionMessageRole.AI:
             if tool_calls is None:
                 return ChatCompletionAssistantMessageParam(
@@ -740,6 +797,8 @@ class AnthropicStreamingClient(PlaygroundStreamingClient):
                     pass
                 elif isinstance(event, anthropic_streaming.InputJsonEvent):
                     raise NotImplementedError
+                elif isinstance(event, anthropic_streaming._types.CitationEvent):
+                    raise NotImplementedError
                 else:
                     assert_never(event)
 
