@@ -4,23 +4,11 @@ import logging
 import re
 import weakref
 from collections import Counter
+from collections.abc import Iterable, Mapping, Sequence
 from datetime import datetime
 from io import BytesIO
 from pathlib import Path
-from typing import (
-    Any,
-    BinaryIO,
-    Dict,
-    Iterable,
-    List,
-    Literal,
-    Mapping,
-    Optional,
-    Sequence,
-    Tuple,
-    Union,
-    cast,
-)
+from typing import Any, BinaryIO, Literal, Optional, Union, cast
 from urllib.parse import quote, urljoin
 
 import httpx
@@ -48,6 +36,7 @@ from phoenix.trace import Evaluations, TraceDataset
 from phoenix.trace.dsl import SpanQuery
 from phoenix.trace.otel import encode_span_to_otlp
 from phoenix.utilities.client import VersionedClient
+from phoenix.utilities.json import decode_df_from_json_string
 
 logger = logging.getLogger(__name__)
 
@@ -129,7 +118,7 @@ class Client(TraceDataExtractor):
         # Deprecated
         stop_time: Optional[datetime] = None,
         timeout: Optional[int] = DEFAULT_TIMEOUT_IN_SECONDS,
-    ) -> Optional[Union[pd.DataFrame, List[pd.DataFrame]]]:
+    ) -> Optional[Union[pd.DataFrame, list[pd.DataFrame]]]:
         """
         Queries spans from the Phoenix server or active session based on specified criteria.
 
@@ -140,9 +129,10 @@ class Client(TraceDataExtractor):
             root_spans_only (bool, optional): If True, only root spans are returned. Default None.
             project_name (str, optional): The project name to query spans for. This can be set
                 using environment variables. If not provided, falls back to the default project.
+           timeout (int, optional): The number of seconds to wait for the server to respond.
 
         Returns:
-            Union[pd.DataFrame, List[pd.DataFrame]]:
+            Union[pd.DataFrame, list[pd.DataFrame]]:
                 A pandas DataFrame or a list of pandas.
                 DataFrames containing the queried span data, or None if no spans are found.
         """
@@ -157,6 +147,7 @@ class Client(TraceDataExtractor):
             end_time = end_time or stop_time
         try:
             response = self._client.post(
+                headers={"accept": "application/json"},
                 url=urljoin(self._base_url, "v1/spans"),
                 params={
                     "project_name": project_name,
@@ -192,14 +183,32 @@ class Client(TraceDataExtractor):
         elif response.status_code == 422:
             raise ValueError(response.content.decode())
         response.raise_for_status()
-        source = BytesIO(response.content)
         results = []
-        while True:
-            try:
-                with pa.ipc.open_stream(source) as reader:
-                    results.append(reader.read_pandas())
-            except ArrowInvalid:
-                break
+        content_type = response.headers.get("Content-Type")
+        if isinstance(content_type, str) and "multipart/mixed" in content_type:
+            if "boundary=" in content_type:
+                boundary_token = content_type.split("boundary=")[1].split(";", 1)[0]
+            else:
+                raise ValueError(
+                    "Boundary not found in Content-Type header for multipart/mixed response"
+                )
+            boundary = f"--{boundary_token}"
+            text = response.text
+            while boundary in text:
+                part, text = text.split(boundary, 1)
+                if "Content-Type: application/json" in part:
+                    json_string = part.split("\r\n\r\n", 1)[1].strip()
+                    df = decode_df_from_json_string(json_string)
+                    results.append(df)
+        else:
+            # For backward compatibility
+            source = BytesIO(response.content)
+            while True:
+                try:
+                    with pa.ipc.open_stream(source) as reader:
+                        results.append(reader.read_pandas())
+                except ArrowInvalid:
+                    break
         if len(results) == 1:
             df = results[0]
             return None if df.shape == (0, 0) else df
@@ -208,7 +217,9 @@ class Client(TraceDataExtractor):
     def get_evaluations(
         self,
         project_name: Optional[str] = None,
-    ) -> List[Evaluations]:
+        *,  # Only support kwargs from now on
+        timeout: Optional[int] = DEFAULT_TIMEOUT_IN_SECONDS,
+    ) -> list[Evaluations]:
         """
         Retrieves evaluations for a given project from the Phoenix server or active session.
 
@@ -216,9 +227,10 @@ class Client(TraceDataExtractor):
             project_name (str, optional): The name of the project to retrieve evaluations for.
                 This can be set using environment variables. If not provided, falls back to the
                 default project.
+            timeout (int, optional): The number of seconds to wait for the server to respond.
 
         Returns:
-            List[Evaluations]:
+            list[Evaluations]:
                 A list of Evaluations objects containing evaluation data. Returns an
                 empty list if no evaluations are found.
         """
@@ -229,6 +241,7 @@ class Client(TraceDataExtractor):
                 "project_name": project_name,
                 "project-name": project_name,  # for backward-compatibility
             },
+            timeout=timeout,
         )
         if response.status_code == 404:
             logger.info("No evaluations found.")
@@ -255,15 +268,18 @@ class Client(TraceDataExtractor):
                 f"with `import phoenix as px; px.launch_app()`"
             )
 
-    def log_evaluations(self, *evals: Evaluations, **kwargs: Any) -> None:
+    def log_evaluations(
+        self,
+        *evals: Evaluations,
+        timeout: Optional[int] = DEFAULT_TIMEOUT_IN_SECONDS,
+        **kwargs: Any,
+    ) -> None:
         """
         Logs evaluation data to the Phoenix server.
 
         Args:
             evals (Evaluations): One or more Evaluations objects containing the data to log.
-            project_name (str, optional): The project name under which to log the evaluations.
-                This can be set using environment variables. If not provided, falls back to the
-                default project.
+            timeout (int, optional): The number of seconds to wait for the server to respond.
 
         Returns:
             None
@@ -282,6 +298,7 @@ class Client(TraceDataExtractor):
                 url=urljoin(self._base_url, "v1/evaluations"),
                 content=cast(bytes, sink.getvalue().to_pybytes()),
                 headers=headers,
+                timeout=timeout,
             ).raise_for_status()
 
     def log_traces(self, trace_dataset: TraceDataset, project_name: Optional[str] = None) -> None:
@@ -426,7 +443,7 @@ class Client(TraceDataExtractor):
             pandas DataFrame
         """
         url = urljoin(self._base_url, f"v1/datasets/{dataset_id}/versions")
-        response = httpx.get(url=url, params={"limit": limit})
+        response = self._client.get(url=url, params={"limit": limit})
         response.raise_for_status()
         if not (records := response.json()["data"]):
             return pd.DataFrame()
@@ -768,10 +785,10 @@ class Client(TraceDataExtractor):
 FileName: TypeAlias = str
 FilePointer: TypeAlias = BinaryIO
 FileType: TypeAlias = str
-FileHeaders: TypeAlias = Dict[str, str]
+FileHeaders: TypeAlias = dict[str, str]
 
 
-def _get_csv_column_headers(path: Path) -> Tuple[str, ...]:
+def _get_csv_column_headers(path: Path) -> tuple[str, ...]:
     path = path.resolve()
     if not path.is_file():
         raise FileNotFoundError(f"File does not exist: {path}")
@@ -788,7 +805,7 @@ def _get_csv_column_headers(path: Path) -> Tuple[str, ...]:
 def _prepare_csv(
     path: Path,
     keys: DatasetKeys,
-) -> Tuple[FileName, FilePointer, FileType, FileHeaders]:
+) -> tuple[FileName, FilePointer, FileType, FileHeaders]:
     column_headers = _get_csv_column_headers(path)
     (header, freq), *_ = Counter(column_headers).most_common(1)
     if freq > 1:
@@ -803,7 +820,7 @@ def _prepare_csv(
 def _prepare_pyarrow(
     df: pd.DataFrame,
     keys: DatasetKeys,
-) -> Tuple[FileName, FilePointer, FileType, FileHeaders]:
+) -> tuple[FileName, FilePointer, FileType, FileHeaders]:
     if df.empty:
         raise ValueError("dataframe has no data")
     (header, freq), *_ = Counter(df.columns).most_common(1)
@@ -824,7 +841,7 @@ _response_header = re.compile(r"(?i)(response|answer|output)s*$")
 
 def _infer_keys(
     table: Union[str, Path, pd.DataFrame],
-) -> Tuple[Tuple[str, ...], Tuple[str, ...], Tuple[str, ...]]:
+) -> tuple[tuple[str, ...], tuple[str, ...], tuple[str, ...]]:
     column_headers = (
         tuple(table.columns)
         if isinstance(table, pd.DataFrame)
